SystemOrganization addCategory: #'Installer-Core'!

Object subclass: #Installer
	instanceVariableNames: 'sm wsm mc ma url user markers password project package bug answers packages messagesToSuppress date'
	classVariableNames: 'Entities'
	poolDictionaries: ''
	category: 'Installer-Core'!

!Installer commentStamp: 'kph 11/18/2006 00:15' prior: 0!
Installer provides a simple Domain Specific Language for installing packages from monticello and squeakmap of various version.

One design decision is to be able to paste scripts a workspace and run and tested from there without any special editing.

Examples - Using SqueakMap

1)
squeakmap := Installer squeakmap.
squeakmap install: 'DynamicBindings'.
squeakmap open.
2)
Installer squeakmap install: 'DynamicBindings'.
3) Alternatively using websqueakmap (similar usage api to squeakmap)
squeakmap := Installer websqueakmap.
squeakmap package: 'DynamicBindings'; install.

4) Abbreviated instanciation
squeakmap := Installer sm.
squeakmap := Installer wsm.

5) Finding Packages by Name
(Installer sm match: 'Labby*') explore.

6) Obtaining List of Package Versions
(Installer sm package: 'Labby & Walker'; versions)  explore.  

6) Searching Packages
Installer sm search: 'seaside'.
Installer sm search: 'author:*Smith'. 
fields: name:, summary:,description:

7) Specify specific version of a package
Installer sm install: 'Labby(17)'.

Examples - Using Monticello
1)
squeaksource := Installer repository: 'http://www.squeaksource.com'.
seaside := squeaksource project: 'Seaside'.
seaside install: 'Comet-lr.8';
		install: 'Scriptaculous'.
2) 
squeaksource := Installer repository: 'http://www.squeaksource.com'		
squeaksource project: 'Seaside';
			  install: 'Comet-lr.8;
			  install: 'Scriptaculous'.
3) Supplying user/password.
squeaksource := Installer repository: 'http://www.squeaksource.com'.
squeaksource user: 'me'; password: 'asecret'.

4) Abbreviated instanciation for convenience.
squeaksource := Installer squeakSource.
or
squeaksource := Installer ss.

Examples - install straight from a url		
1a)
(Installer url: 'http://minnow.cc.gatech.edu/squeak/uploads/5889/MakeTestsGreen39.cs') install.
1b)
Installer installUrl: 'http://minnow.cc.gatech.edu/squeak/uploads/5889/MakeTestsGreen39.cs'.

2)
| page |
page := Installer url: 'http://minnow.cc.gatech.edu/squeak/uploads/5889/'.
page package: 'MakeTestsGreen39.cs'.
page install.

3)
In html page scripts are delimited by <code st> </code st>
| page |
page := Installer url: 'http://wiki.squeak.org/742'.
page install.
4)
specify your own delimeters
| page |
page := Installer url: 'http://wiki.squeak.org/742'.
page markers: 'beginning of script...end of script'.
page install.

Note: Scripts embedded in html or a swiki page may need to escape some entities.
Supported entities are &amp; &gt; &lt; &star; (see Installer-c-entities)


!

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 12/18/2006 02:42'!
entities

^ Entities ifNil: [ Entities := 
"enough entities to be going on with"
  Dictionary new.
Entities at: 'lt' put: '<';
	at: 'gt' put: '>';
	at: 'amp' put: '&';
	at: 'star' put: '*';
	at: 'quot' put: '''';
 	yourself
]

 ! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 11/7/2006 04:24'!
installUrl: urlString

^(self url: urlString) install! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 10/19/2006 06:55'!
mantis

^self mantis: 'http://bugs.impara.de/'! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 10/19/2006 06:56'!
mantis: host

^self new ma: host; yourself! !

!Installer class methodsFor: 'accessing' stamp: 'kph 11/12/2006 16:37'!
repository: host  

^self new mc: host ! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 11/12/2006 16:30'!
sm

^self squeakmap! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 10/15/2006 06:37'!
squeakmap

^self new sm: true; yourself! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 11/17/2006 01:52'!
squeaksource

^self repository: 'http://www.squeaksource.com'! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 11/17/2006 01:53'!
ss

^self squeaksource
 ! !

!Installer class methodsFor: 'accessing' stamp: 'kph 11/10/2006 00:48'!
url: urlString

^self new url: urlString; yourself! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 10/21/2006 01:38'!
websqueakmap

^self new wsm: 'http://map.squeak.org'; yourself! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 10/21/2006 01:38'!
websqueakmap: host

^self new wsm: host; yourself! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 11/12/2006 16:35'!
wsm

^self websqueakmap! !

!Installer methodsFor: 'auto answering' stamp: 'kph 10/17/2006 13:07'!
answer: aString with: anAnswer

^self answers add: ( Array with: aString with: anAnswer )! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/17/2006 13:06'!
answers

^ answers ifNil: [ answers := OrderedCollection new ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 06:55'!
answers: anObject
	"Set the value of answers"

	answers := anObject! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 05:06'!
bug
	"Answer the value of bug"

	^ bug! !

!Installer methodsFor: 'mantis' stamp: 'kph 10/19/2006 12:21'!
bug: aBugNo

^ (self maFilesFor: aBugNo) keys! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 02:26'!
bug: aBugNo fix: aFileName

	| stream |

	self logCR: 'obtaining ', aFileName, '...'.

	stream := self maStreamFor: aBugNo file: aFileName.

	self install: aFileName fromStream: stream.
	
 	^ date! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 02:29'!
bug: aBugNo fix: aFileName date: aDate

	 self bug: aBugNo fix: aFileName.
	
	aDate ~= date ifTrue: [ self error: 'bug ', aBugNo asString, ' updated since ', aDate ].
	
	^ date! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 02:10'!
bug: aBugNo retrieve: aFileName

	^ (self maStreamFor: aBugNo file: aFileName) contents! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 22:17'!
classChangeSet

^Smalltalk at: #ChangeSet  ifAbsent: [ self error: 'ChangeSet not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 22:16'!
classGZipReadStream

^Smalltalk at: #GZipReadStream  ifAbsent: [ self error: 'Compression not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 15:42'!
classHTTPSocket

^Smalltalk at: #HTTPSocket ifAbsent: [ self error: 'Network package not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/17/2006 03:15'!
classMCHttpRepository

^Smalltalk at: #MCHttpRepository ifAbsent: [ self error: 'Monticello not present' ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 11/17/2006 02:46'!
classMCMczReader

^Smalltalk at: #MCMczReader ifAbsent: [ nil ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 11/17/2006 02:44'!
classMczInstaller

^Smalltalk at: #MczInstaller ifAbsent: [ nil ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 11/23/2006 03:18'!
classMultiByteBinaryOrTextStream
^Smalltalk at: #MultiByteBinaryOrTextStream  ifAbsent: [ self error: 'MultiByteBinaryOrTextStream not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 15:40'!
classSARInstaller

^Smalltalk at: #SARInstaller  ifAbsent: [ self error: 'SARInstaller not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 18:46'!
classSMLoader

^Smalltalk at: #SMLoader  ifAbsent: [ self error: 'SqueakMap not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 15:40'!
classSMSqueakMap

^Smalltalk at: #SMSqueakMap  ifAbsent: [ self error: 'SqueakMap not present' ]! !

!Installer methodsFor: 'url' stamp: 'kph 11/10/2006 01:18'!
extractFromHtml: html

	| source in start stop  |
 
	start := self markers copyUpTo: $..
	stop := self markers copyAfterLast: $..
		 
	html upToAll: start.
	in := (html upToAll: stop) readStream.  
	"ignore any tags"
	source := ReadWriteStream on: (String new: 100).
	[ in atEnd ] whileFalse: [ 
		source nextPutAll: (in upTo: $<).
		in upTo: $>.	
	].
	in := source reset.
	"perform html entity substitution"
	source := ReadWriteStream on: (String new: 100).
	[ in atEnd ] whileFalse: [ 
		source nextPutAll: (in upTo: $&).
		in atEnd ifFalse: [ source nextPutAll: (self class entities at: (in upTo: $;) ifAbsent: '?') ].	
	].
	^source reset   
! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 02:53'!
fixBug: aBugNo

  ^(self class url: (self maPageFor: aBugNo)) markers: '{fix}...{/fix}'; install.
	! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 02:57'!
fixBugs: theBugNos

theBugNos do: [:bugNo | Installer mantis fixBug: bugNo ] ! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/12/2006 16:16'!
info

self sm ifTrue: [ ^ self smInfo  ].
self wsm ifNotNil: [ ^ self wsmInfo  ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/10/2006 00:14'!
install

self logErrorDuring: [
	self mc ifNotNil: [ self mcInstall ].
	self wsm ifNotNil: [ self wsmInstall ].
	self url ifNotNil: [ self urlInstall ].
	self sm ifTrue: [ self smInstall ].
] ! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/18/2006 11:29'!
install: packageNameCollectionOrDetectBlock

self package: packageNameCollectionOrDetectBlock.

self install! !

!Installer methodsFor: 'mantis' stamp: 'kph 11/16/2006 22:20'!
install: aFileName fromStream: stream

| ext installSelector |
	 
	self log: 'installing...'.
 
	ext := aFileName copyAfterLast: $..
	installSelector := ('install', ext asUppercase, ':fromStream:') asSymbol.
	
	self withAnswersDo:	[
		(self respondsTo: installSelector)
			ifTrue: [ self perform: installSelector with: aFileName with: stream ]
			ifFalse: [ stream fileIn ].
	]. 

	self log: '.done'
 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 11/16/2006 22:18'!
installCS: aFileName fromStream: stream 
 
	self classChangeSet newChangesFromStream: stream named:aFileName.
! !

!Installer methodsFor: 'mantis' stamp: 'kph 11/23/2006 03:22'!
installGZ: aFileName fromStream: stream 

	"FileIn the contents of a gzipped stream"

	| zipped unzipped |
	 
	zipped := self classGZipReadStream on: stream.
	unzipped := MultiByteBinaryOrTextStream with: zipped contents asString.
	unzipped reset.

	self  newChangeSetFromStream: unzipped named:aFileName.

	! !

!Installer methodsFor: 'mantis' stamp: 'kph 11/17/2006 02:53'!
installMCZ: aFileName fromStream: stream 

	self classMczInstaller ifNotNilDo: [ :reader | ^reader installStream: stream].  
	self classMCMczReader ifNotNilDo: [ :reader | ^(reader versionFromStream: stream) load]. 

	self error: 'no monticello readers available'. 
 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 11/16/2006 22:18'!
installSAR: aFileName fromStream: stream 

 | newCS |

	self classSARInstaller withCurrentChangeSetNamed: aFileName
		do: [:cs | newCS := cs. self classSARInstaller new fileInFrom: stream].
	newCS isEmpty ifTrue: [ self classChangeSet removeChangeSet: newCS ]! !

!Installer methodsFor: 'logging' stamp: 'kph 10/17/2006 12:13'!
log: text

Transcript show: text.! !

!Installer methodsFor: 'logging' stamp: 'kph 10/17/2006 12:14'!
logCR: text

Transcript cr; show: text.! !

!Installer methodsFor: 'logging' stamp: 'kph 11/10/2006 00:16'!
logErrorDuring: block

block on: Error do: [ :e | self logCR: e messageText ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 06:55'!
ma
	"Answer the value of ma"

	^ ma! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 07:07'!
ma: aUrl
 
	"Set the value of host"

	ma := aUrl last = $/ ifTrue: [ aUrl ] ifFalse: [ aUrl, '/' ]! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 02:38'!
maFilesFor: maBugNo
  "
  self mantis bug: 5251.
"
 
 | file files bugPage id  | 
 
 files := Dictionary new.
 
 bugPage := self maPageFor: maBugNo.

 date := (bugPage upToAll: '<!!-- Date Updated -->'; upToAll: '<td>'; upToAll: '</td>') withBlanksTrimmed upTo: $ .
 [
  id := bugPage upToAll: 'href="file_download.php?file_id='; upTo: $&. 
 file := bugPage upToAll: 'amp;type=bug"' ; upTo: $<.
 
 ((file size > 1) and: [file first = $>]) ifTrue: [ files at: file copyWithoutFirst put: id ].
 
 id notEmpty ] whileTrue.

^files ! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 02:37'!
maPageFor: maBugNo
  "
  self mantis bug: 5251.
 "
 
^ HTTPSocket httpGet: (self ma, 'view.php?id=', maBugNo asString).
 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 02:51'!
maStreamFor: aBugNo file: aFileName

	| fileId  |

 	fileId :=  (self maFilesFor: aBugNo) at: aFileName.

 	^ HTTPSocket httpGet: (self ma, 'file_download.php?file_id=' , fileId , '&type=bug').
	 ! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 06:19'!
markers
	"Answer the value of markers"

	^ markers ifNil: [ '<code st>...</code st>' ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 05:06'!
markers: anObject
	"Set the value of markers"

	markers := anObject! !

!Installer methodsFor: 'searching' stamp: 'kph 11/12/2006 16:11'!
match: aMatch

^self packagesMatching: aMatch! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:38'!
mc
	"Answer the value of mc"

	^ mc! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 07:07'!
mc: aUrl
 
	"Set the value of host"

	mc := aUrl last = $/ ifTrue: [ aUrl ] ifFalse: [ aUrl, '/' ]! !

!Installer methodsFor: 'monticello' stamp: 'kph 11/17/2006 03:16'!
mcInstall

	| repository sortMczs files fileToLoad  version detectFileBlock  count |

	self logCR: 'installing ', self package asString, '...'.

	self package isString ifTrue: [ detectFileBlock := [ :file | file beginsWith: self package ] ].
	(self package isKindOf: Array) 
			ifTrue: [ detectFileBlock :=  [ :file | (self package detect: [ :item | file beginsWith: item ] ifNone: [ false ]) ~= false ] ].
	self package isBlock ifTrue: [ detectFileBlock := self package ].
  

	repository := self mcRepository.

	sortMczs := [:a :b | 
        	[(a findBetweenSubStrs: #($.)) allButLast last asInteger > (b findBetweenSubStrs: #($.)) allButLast last asInteger] on: Error do: [:ex | false]].

	"several attempts to read files - repository readableFileNames sometimes fails"

	count := 0. fileToLoad := nil.
	
	[count := count + 1.
	 (fileToLoad = nil) and:[ count < 5 ] ] 
		whileTrue: [
						files := repository readableFileNames asSortedCollection: sortMczs.
						fileToLoad := files detect: detectFileBlock ifNone: [ nil ].
																						].

	version := repository versionFromFileNamed: fileToLoad.
	version workingCopy repositoryGroup addRepository: repository.
	repository creationTemplate: 'MCHttpRepository
        	location: ''', self mcUrl, '''
        	user: ''', self user, '''
        	password: ''', self password, ''''.
	self log: ' ', version fileName, '...'.

	self withAnswersDo: [ version load ].

	self log: 'done'.
! !

!Installer methodsFor: 'monticello' stamp: 'kph 11/16/2006 17:47'!
mcRepository

^self classMCHttpRepository location: (self mcUrl) user: self user password: self password.! !

!Installer methodsFor: 'monticello' stamp: 'kph 12/17/2006 17:58'!
mcUrl

^ self mc, (self project ifNil: [''])! !

!Installer methodsFor: 'monticello' stamp: 'kph 10/21/2006 05:47'!
mcVersions

^ (self packages select: [ :p | ( self package,'-*.mcz' ) match: p ]) collect: [ :p | self copy package: p  ; yourself ].
 ! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/17/2006 14:30'!
messagesToSuppress

^ messagesToSuppress ifNil: [ messagesToSuppress := OrderedCollection new ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 06:55'!
messagesToSuppress: anObject
	"Set the value of messagesToSuppress"

	messagesToSuppress := anObject! !

!Installer methodsFor: 'mantis' stamp: 'kph 11/23/2006 03:24'!
newChangeSetFromStream: aStream named: aName 

	"This code is based upon ChangeSet-c-#newChangesFromStream:named: which is in 3.9,
	implemented here for previous versions"

	| oldChanges newName newSet newStream |

	(self classChangeSet respondsTo: #newChangesFromStream:named:) 
		ifTrue: [ ^self classChangeSet newChangesFromStream: aStream named:aName ].


	oldChanges := ChangeSet current.
 
	"so a Bumper update can find it"
	newName := aName sansPeriodSuffix.

	newSet := self classChangeSet basicNewNamed: newName.

	[newSet
		ifNotNil: [(aStream respondsTo: #converter:)
				ifTrue: [newStream := aStream]
				ifFalse: [newStream := self classMultiByteBinaryOrTextStream with: aStream contentsOfEntireFile.
					newStream reset].
			self classChangeSet newChanges: newSet.
			newStream setConverterForCode.
			newStream fileInAnnouncing: 'Loading ' , newName , '...'.
			Transcript cr; show: 'File ' , aName , ' successfully filed in to change set ' , newName].
	aStream close]
		ensure: [self classChangeSet newChanges: oldChanges].
	 
	^ newSet! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 07:59'!
open
	
self sm ifTrue: [ SMLoader open ].
self mc ifNotNil: [self mcRepository morphicOpen: nil ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:05'!
package
	"Answer the value of package"

	^ package! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:05'!
package: anObject
	"Set the value of package"

	package := anObject! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/12/2006 16:23'!
packages
	
self sm ifTrue: [ ^self smPackages ].
self mc ifNotNil: [ ^self mcRepository allFileNames ].
self wsm ifNotNil: [ ^self wsmPackagesByName keys ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 05:06'!
packages: anObject
	"Set the value of packages"

	packages := anObject! !

!Installer methodsFor: 'searching' stamp: 'kph 11/12/2006 23:19'!
packagesMatching: aMatch

self sm ifTrue: [ ^ (self packages select: [ :p | aMatch match: p name ]) collect: [ :p | self copy package: p name; yourself ] ].
self mc ifNotNil: [ ^ (self packages select: [ :p | ( aMatch , '.mcz' ) match: p ]) collect: [ :p | self copy package: p ; yourself ] ].
self wsm ifNotNil: [ ^ (self packages select: [ :p | ( aMatch) match: p ]) collect: [ :p | self copy package: p ; yourself ] ].

^'search type not supported'! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:16'!
password
	"Answer the value of password"

	^ password  ifNil: [ '' ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 03:56'!
password: anObject
	"Set the value of password"

	password := anObject! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/8/2006 00:56'!
printOn: s
 
s nextPutAll: '(Installer'.

self sm ifTrue: [ s nextPutAll: ' squeakmap' ].
self ma ifNotNil: [ s nextPutAll: ' mantis' ].
self wsm ifNotNil: [ s nextPutAll: ' websqueakmap' ].
self url ifNotNil: [ s nextPutAll: ' url:''', self url,'''' ].
self mc ifNotNil: [ s nextPutAll: ' repository:''', self mc,'''' ].

s nextPut: $).

self project ifNotNil: [ s nextPutAll: ' project:';
  						nextPutAll: '''', self project, ''''.
					   self package ifNotNil: [ s nextPutAll: '; '] ].
					
self package ifNotNil: [ s nextPutAll: ' package:';
  						 nextPutAll: '''', self package asString, '''' ].
					
s nextPut: $..! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:30'!
project
	"Answer the value of project"

	^ project! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:33'!
project: anObject
	"Set the value of project"

	project := anObject.

	^self copy.! !

!Installer methodsFor: 'searching' stamp: 'kph 11/12/2006 23:19'!
search: aMatch

self sm ifTrue: [ ^ self smSearch: ('*',aMatch,'*') ].
self mc ifTrue: [ ^ self mcSearch: ('*',aMatch,'*')  ].
self wsm ifNotNil: [ ^ self wsmSearch: ('*',aMatch,'*')  ].

^'search type not supported'! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:55'!
sm
	"Answer the value of sm"

	^ sm ifNil: [ false ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:38'!
sm: anObject
	"Set the value of sm"

	sm := anObject! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 10/21/2006 06:54'!
smInstall 

	| pkgAndVersion releases release |

	pkgAndVersion := self smPackageAndVersion.

	self logCR: 'installing ', self package, ' from SqueakMap...'.


	releases := self smReleasesForPackage: pkgAndVersion first.
 	
	release := pkgAndVersion last isEmpty ifTrue: [ releases last ]
					ifFalse:[ releases detect: [ :rel | rel version = pkgAndVersion last ] ]. 
	
	self withAnswersDo: [ release install ].

	self log: ' done'.
! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 10/17/2006 15:24'!
smPackageAndVersion

| p |

p := ReadStream on: self package .

^Array with: (p upTo: $() with: (p upTo: $)).! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 11/16/2006 18:46'!
smPackages
	
^self classSMLoader new packages! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 11/16/2006 15:40'!
smReleasesForPackage: name 

^(self classSMSqueakMap default packageWithName: name) releases! !

!Installer methodsFor: 'searching' stamp: 'kph 11/12/2006 23:28'!
smSearch: aMatch  

	| results |
	
	results := Set new.

	self packages do: [ :pkg |
		({ 'name:',pkg name.
		   'summary:', pkg summary.
		   'description:', pkg description.
		   'author:', pkg author. } anySatisfy: [ :field | aMatch match: field ])
		 ifTrue: [ results add: (self copy package: pkg name) ]. 
	].

	^results

! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 11/12/2006 16:54'!
smVersions
 
^ (self smReleasesForPackage: self package) collect: [ :v | self copy package: (v package name,'(',v version,')'); yourself. ] 

 ! !

!Installer methodsFor: 'auto answering' stamp: 'kph 10/17/2006 14:30'!
suppress: aMessage

messagesToSuppress add: aMessage! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/17/2006 15:37'!
unload: workingCopyPackageName 

self logCR: 'Unloading ', workingCopyPackageName.
 (MCWorkingCopy forPackage: (MCPackage new name: workingCopyPackageName)) unload! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 04:23'!
url

^url! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 04:23'!
url: aUrl
 
 url := aUrl! !

!Installer methodsFor: 'url' stamp: 'kph 11/16/2006 19:37'!
urlGet

	| page |

	page := HTTPSocket httpGet: self urlToDownload accept: 'application/octet-stream'.  
 
	(page respondsTo: #reset)  ifFalse: [ self error: 'unable to contact web site' ].
 
	((page next: 14) asUppercase = '<!!DOCTYPE HTML') ifTrue: [ page := self extractFromHtml: page ].

	^ page reset
	! !

!Installer methodsFor: 'url' stamp: 'kph 11/8/2006 01:00'!
urlInstall 

"
Installer installUrl: 'wiki.squeak.org/742'.
"
 	
	self logCR: 'found ', self urlToDownload , ' ...'.
	 
	self install:  self urlToDownload fromStream: self urlGet 

	! !

!Installer methodsFor: 'url' stamp: 'kph 11/10/2006 00:47'!
urlToDownload

	^ (self url, (self package ifNil: [ '' ])) asUrl asString.
	
 ! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:15'!
user
	"Answer the value of user"

	^ user ifNil: [ '' ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 03:56'!
user: anObject
	"Set the value of user"

	user := anObject! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/21/2006 05:50'!
versions
	
self sm ifTrue: [ ^self smVersions ].
self mc ifNotNil: [ ^self mcVersions ].
self wsm ifNotNil: [ ^self wsmVersions ].! !

!Installer methodsFor: 'monticello' stamp: 'kph 10/21/2006 06:52'!
withAnswersDo: aBlock

	(aBlock respondsTo: #valueSuppressingMessages:supplyingAnswers: )
		ifTrue: [aBlock valueSuppressingMessages: self messagesToSuppress supplyingAnswers: self answers.]
		ifFalse: [ aBlock value ]
! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 10/21/2006 01:37'!
wsm
	"Answer the value of wsm"
	
	^ wsm! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 10/21/2006 01:37'!
wsm: aUrl
 
	"Set the value of host"

	wsm := aUrl last = $/ ifTrue: [ aUrl ] ifFalse: [ aUrl, '/' ]! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 11/16/2006 15:43'!
wsmDownloadUrl 

| pkgAndVersion packageId packageName packageVersion releaseAutoVersion
 downloadPage |

	pkgAndVersion := self smPackageAndVersion.
	packageName := pkgAndVersion first.
	packageVersion := pkgAndVersion last.
	packageVersion isEmpty ifTrue: [ packageVersion := #latest ].

	packageId := self wsmPackagesByName at: packageName.
	releaseAutoVersion := (self wsmReleasesFor: packageId) at: packageVersion.
					 
	downloadPage := self classHTTPSocket httpGet: (self wsm,'packagebyname/', packageName,'/autoversion/', releaseAutoVersion,'/downloadurl') asUrl asString.
				 		 
	^ downloadPage contents
	
! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 11/16/2006 15:42'!
wsmInstall 

	| downloadUrl |

	self logCR: 'finding ', self package, ' from websqueakmap(', self wsm, ') ...'.

	downloadUrl := self wsmDownloadUrl.
	
	self logCR: 'found at ', downloadUrl asString, ' ...'.
	 
	self install: downloadUrl fromStream: (self classHTTPSocket httpGet: downloadUrl).

	! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 11/16/2006 15:42'!
wsmPackagesByName

	| html id name | 
	
	packages ifNotNil: [ ^packages ].
	
	packages := Dictionary new.
	
	html := self classHTTPSocket httpGet: (self wsm, 'packagesbyname').
	
	[
		id := html upToAll: '/package/'; upToAll: '">'.
		name := html upTo: $<.
		
		(id notEmpty and: [ name notEmpty ]) 

	] whileTrue: [ packages at: name put: id ].

	^ packages	
	! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 11/16/2006 15:42'!
wsmReleasesFor: packageId

	| html autoVersion version releases |
		
	releases := Dictionary new.
	
	html := self classHTTPSocket httpGet: (self wsm, '/package/', packageId ).
	
	[
		releases at: #latest put: autoVersion.
		autoVersion := html upToAll: '/autoversion/'; upTo: $".
		version := html upTo: $-; upTo: $<.
		
		(autoVersion notEmpty and: [version notEmpty ]) 

	] whileTrue: [ releases at: version put: autoVersion ].

	^ releases
	! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 10/21/2006 06:04'!
wsmVersions

| pkgAndVersion packageId packageName packageVersion versions |

	pkgAndVersion := self smPackageAndVersion.
	packageName := pkgAndVersion first.
	packageVersion := pkgAndVersion last.
	packageVersion isEmpty ifTrue: [ packageVersion := #latest ].

	packageId := self wsmPackagesByName at: packageName.
	
	versions := (self wsmReleasesFor: packageId) keys.
	versions remove: #latest.
	
	^ versions collect: [ :version | self copy package: (packageName,'(', version ,')'); yourself ]. ! !
